'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createServer;

var _stages = require('./stages');

var _stages2 = _interopRequireDefault(_stages);

var _expressServer = require('./express-server');

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _network = require('./network');

var Network = _interopRequireWildcard(_network);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultOptions = {
  port: 3000
};

var builtinAdminCommands = {
  start: function start(monsterr) {
    monsterr.start();
  },
  next: function next(monsterr) {
    monsterr.getStageManager().next();
  },
  reset: function reset(monsterr) {
    monsterr.getStageManager().reset();
  },
  players: function players(monsterr) {
    monsterr.send('_msg', monsterr.getNetwork().getPlayers().join(', ')).toAdmin();
  }
};

var builtinEvents = {
  _msg: function _msg(monsterr, clientId, msg) {
    monsterr.send('_msg', msg).toNeighboursOf(clientId);
  },
  _log: function _log(monsterr, _, json) {
    monsterr.log(json.msg, json.fileOrExtra, json.extra);
  },
  _stage_finished: function _stage_finished(monsterr, clientId, stageNo) {
    monsterr.getStageManager().playerFinishedStage(clientId, stageNo);
  }
};

function createServer() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$network = _ref.network,
      network = _ref$network === undefined ? Network.pairs(16) : _ref$network,
      _ref$logger = _ref.logger,
      logger = _ref$logger === undefined ? (0, _logger2.default)({}) : _ref$logger,
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options,
      _ref$events = _ref.events,
      events = _ref$events === undefined ? {} : _ref$events,
      _ref$commands = _ref.commands,
      commands = _ref$commands === undefined ? {} : _ref$commands,
      _ref$adminCommands = _ref.adminCommands,
      adminCommands = _ref$adminCommands === undefined ? {} : _ref$adminCommands,
      _ref$stages = _ref.stages,
      stages = _ref$stages === undefined ? [] : _ref$stages;

  var stageManager = void 0;
  var socketServer = (0, _expressServer.createSocketServer)();

  options = Object.assign(defaultOptions, options);

  function send(topic, message) {
    var event = { type: topic, payload: message };
    return {
      toAll: function toAll() {
        socketServer.sendEvent(event).toAll();
      },
      toClient: function toClient(clientId) {
        socketServer.sendEvent(event).toClients([clientId]);
      },
      toNeighboursOf: function toNeighboursOf(clientId) {
        socketServer.sendEvent(event).toClients([clientId].concat(network.getNeighbours(clientId)));
      },
      toNeighboursOfExclusive: function toNeighboursOfExclusive(clientId) {
        socketServer.sendEvent(event).toClients(network.getNeighbours(clientId));
      },
      toClients: function toClients() {
        var clients = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

        socketServer.sendEvent(event).toClients(clients);
      },
      toAdmin: function toAdmin() {
        socketServer.sendEvent(event).toAdmin();
      }
    };
  }

  function run() {
    (0, _expressServer.createHttpServer)({ port: options.port });
    stageManager = (0, _stages2.default)({
      getContext: function getContext() {
        return monsterr;
      },
      getPlayers: function getPlayers() {
        return network.getPlayers();
      },
      onStageStarted: function onStageStarted(stageNo) {
        return monsterr.send('_start_stage', stageNo).toAll();
      },
      onStageEnded: function onStageEnded(stageNo) {
        return monsterr.send('_end_stage', stageNo).toAll();
      },
      onGameOver: function onGameOver() {
        return monsterr.send('_game_over').toAll();
      },
      stages: stages
    });
  }

  function start() {
    stageManager.start();
  }

  function log(msg, fileOrExtra, extra) {
    logger.log(msg, fileOrExtra, extra);
  }

  function handleCommand(_ref2) {
    var type = _ref2.type,
        args = _ref2.args,
        clientId = _ref2.clientId;

    console.log('CMD:', type, args, clientId);

    // no clientID means it came from admin
    if (!clientId) {
      builtinAdminCommands[type] && builtinAdminCommands[type].apply(builtinAdminCommands, [monsterr].concat(_toConsumableArray(args)));

      adminCommands[type] && adminCommands[type].apply(adminCommands, [monsterr].concat(_toConsumableArray(args)));
    }

    commands[type] && commands[type].apply(commands, [monsterr, clientId].concat(_toConsumableArray(args)));

    var stageCommands = stageManager.getCommands();
    stageCommands[type] && stageCommands[type].apply(stageCommands, [monsterr, clientId].concat(_toConsumableArray(args)));
  }

  function handleEvent(_ref3) {
    var type = _ref3.type,
        payload = _ref3.payload,
        clientId = _ref3.clientId;

    console.log('EVENT:', { type: type, payload: payload, clientId: clientId });
    // check builtin
    builtinEvents[type] && builtinEvents[type](monsterr, clientId, payload);

    // check provided
    events[type] && events[type](monsterr, clientId, payload);

    var stageEvents = stageManager.getEvents();
    stageEvents[type] && stageEvents[type](monsterr, clientId, payload);
  }

  socketServer.on('cmd', function (cmd) {
    return handleCommand(cmd);
  });
  socketServer.on('event', function (event) {
    return handleEvent(event);
  });
  socketServer.on('connect', function (player) {
    console.log(player, 'connected!');
    network.addPlayer(player);
  });
  socketServer.on('disconnect', function (player) {
    console.log(player, 'disconnected!');
    network.removePlayer(player);
    stageManager.playerDisconnected(player);
  });

  /** API */
  var monsterr = {
    run: run,
    start: start,
    send: send,
    log: log,

    getNetwork: function getNetwork() {
      return network;
    },
    getStageManager: function getStageManager() {
      return stageManager;
    },
    getCommands: function getCommands() {
      return commands;
    },
    getEvents: function getEvents() {
      return events;
    }
  };

  return monsterr;
}