'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createServer;

var _httpServer = require('./http-server');

var _httpServer2 = _interopRequireDefault(_httpServer);

var _socketServer = require('./socket-server');

var _socketServer2 = _interopRequireDefault(_socketServer);

var _serverStageManager = require('./server-stage-manager');

var _serverStageManager2 = _interopRequireDefault(_serverStageManager);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _network = require('./network');

var Network = _interopRequireWildcard(_network);

var _util = require('../util');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultOptions = {
  port: 3000
};

var builtinAdminCommands = {
  start: function start(monsterr) {
    monsterr.start();
  },
  next: function next(monsterr) {
    monsterr.getStageManager().next();
  },
  reset: function reset(monsterr) {
    monsterr.getStageManager().reset();
  },
  players: function players(monsterr) {
    monsterr.send('_msg', monsterr.getNetwork().getPlayers().join(', ')).toAdmin();
  },
  latencies: function latencies(monsterr) {
    monsterr.send('_msg', JSON.stringify(monsterr.getLatencies())).toAdmin();
  }
};

var builtinEvents = {
  _set_name: function _set_name(monsterr, clientId, name) {
    monsterr.setName(clientId, name);
  },
  _msg: function _msg(monsterr, clientId, msg) {
    monsterr.send('_msg', { msg: msg, name: monsterr.getName(clientId) }).toNeighboursOf(clientId);
  },
  _log: function _log(monsterr, _, json) {
    monsterr.log(json.msg, json.fileOrExtra, json.extra);
  },
  _stage_finished: function _stage_finished(monsterr, clientId, stageNo) {
    monsterr.getStageManager().playerFinishedStage(clientId, stageNo);
  }
};

function createServer() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$network = _ref.network,
      network = _ref$network === undefined ? Network.pairs(16) : _ref$network,
      _ref$logger = _ref.logger,
      logger = _ref$logger === undefined ? (0, _logger2.default)({}) : _ref$logger,
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options,
      _ref$events = _ref.events,
      events = _ref$events === undefined ? {} : _ref$events,
      _ref$commands = _ref.commands,
      commands = _ref$commands === undefined ? {} : _ref$commands,
      _ref$adminCommands = _ref.adminCommands,
      adminCommands = _ref$adminCommands === undefined ? {} : _ref$adminCommands,
      _ref$stages = _ref.stages,
      stages = _ref$stages === undefined ? [] : _ref$stages;

  var httpServer = (0, _httpServer2.default)({ port: options.port });
  var socketServer = (0, _socketServer2.default)(httpServer.getIO());
  var nameMap = {};
  var stageManager = void 0;

  options = Object.assign(defaultOptions, options);

  function send(topic, message) {
    var event = { type: topic, payload: message };
    return {
      toAll: function toAll() {
        socketServer.sendEvent(event).toAll();
      },
      toClient: function toClient(clientId) {
        socketServer.sendEvent(event).toClients([clientId]);
      },
      toNeighboursOf: function toNeighboursOf(clientId) {
        socketServer.sendEvent(event).toClients([clientId].concat(network.getNeighbours(clientId)));
      },
      toNeighboursOfExclusive: function toNeighboursOfExclusive(clientId) {
        socketServer.sendEvent(event).toClients(network.getNeighbours(clientId));
      },
      toClients: function toClients() {
        var clients = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

        socketServer.sendEvent(event).toClients(clients);
      },
      toAdmin: function toAdmin() {
        socketServer.sendEvent(event).toAdmin();
      }
    };
  }

  function run() {
    stageManager = (0, _serverStageManager2.default)({
      getContext: function getContext() {
        return monsterr;
      },
      getPlayers: function getPlayers() {
        return network.getPlayers();
      },
      onStageStarted: function onStageStarted(stageNo) {
        return monsterr.send('_start_stage', stageNo).toAll();
      },
      onStageEnded: function onStageEnded(stageNo) {
        return monsterr.send('_end_stage', stageNo).toAll();
      },
      onGameOver: function onGameOver() {
        return monsterr.send('_game_over').toAll();
      },
      stages: stages
    });
  }

  function start() {
    stageManager.start();
  }

  function log(msg, fileOrExtra, extra) {
    logger.log(msg, fileOrExtra, extra);
  }

  socketServer.on('cmd', function (cmd) {
    return (0, _util.handleCommand)(cmd, [commands, stageManager.getCommands(), !cmd.clientId ? builtinAdminCommands : {}, !cmd.clientId ? adminCommands : {}], monsterr);
  });
  socketServer.on('event', function (event) {
    return (0, _util.handleEvent)(event, [events, builtinEvents, stageManager.getEvents()], monsterr);
  });

  var resumeCurrentStage = function resumeCurrentStage(player) {
    var currentStage = stageManager.getCurrentStage();
    if (options.resumeCurrentStage && currentStage !== -1) {
      setTimeout(function () {
        return monsterr.send('_start_stage', currentStage).toClient(player);
      }, 200);
    }
  };

  socketServer.on('reconnect', function (player) {
    console.log(player, 'reconnected!');
    resumeCurrentStage(player);
  });
  socketServer.on('connect', function (player) {
    console.log(player, 'connected!');
    network.addPlayer(player);
    resumeCurrentStage(player);
  });
  socketServer.on('disconnect', function (player) {
    console.log(player, 'disconnected!');
    network.removePlayer(player);
    stageManager.playerDisconnected(player);
  });

  /** API */
  var monsterr = {
    run: run,
    start: start,
    send: send,
    log: log,

    getNetwork: function getNetwork() {
      return network;
    },
    getStageManager: function getStageManager() {
      return stageManager;
    },
    getCommands: function getCommands() {
      return commands;
    },
    getEvents: function getEvents() {
      return events;
    },
    getLatencies: function getLatencies() {
      return socketServer.getLatencies();
    },

    setName: function setName(id, name) {
      var prevName = monsterr.getName(id);
      nameMap[id] = name;
      monsterr.send('_set_name', { id: id, name: name, prevName: prevName }).toClient(id);
      monsterr.send('_rename', { id: id, name: name, prevName: prevName }).toNeighboursOf(id);
    },
    getName: function getName(id) {
      return nameMap[id] || id;
    }
  };

  return monsterr;
}