'use strict';

var uniq = require('lodash/uniq');

/**
 * @typedef {number[][]} adjList
 */

/**
 * Returns the first empty (undefined) index in an array or -1 if no empty indexes exist.
 * @param {*[]} array the array to check
 * @returns {number}
 */
function firstEmptyIndex(array) {
  return array.reduce(function (prev, elem, index) {
    if (prev !== -1) {
      return prev;
    }

    return elem === undefined ? index : -1;
  }, -1);
}

/**
 * Returns an adjecency list representing the same network layout as the matrix.
 * @param {adjList} matrix The matrix to convert.
 * @returns {adjList}
 */
function matrixToList(matrix) {
  return matrix.map(function (row, rowIndex) {
    return row.map(function (e, i) {
      return e !== 0 ? i : -1;
    }).filter(function (elem, colIndex) {
      return elem !== -1 && colIndex !== rowIndex;
    });
  });
}

/**
 * Returns an adjecency matrix representing the same network layout as the list.
 * @param {adjList} list The list to convert.
 * @returns {adjList}
 */
function listToMatrix(list) {
  return list.map(function (row, rowIndex) {
    var newRow = [];
    for (var colIndex = 0; colIndex < list.length; colIndex++) {
      if (colIndex === rowIndex || row.includes(colIndex)) {
        newRow.push(1);
      } else {
        newRow.push(0);
      }
    }
    return newRow;
  });
}

/**
 * Returns a network.
 * @param {adjList} adjList an adjecency list defining a network layout
 */
function createNetwork(adjList) {
  // remove dupes
  adjList = adjList.map(uniq);

  var players = [];
  var playerToIndex = {};

  return {
    addPlayer: function addPlayer(player) {
      var nextIndex = firstEmptyIndex(players);
      if (nextIndex !== -1) {
        playerToIndex[player] = nextIndex;
        players[nextIndex] = player;
      } else {
        if (players.length === adjList.length) {
          return false;
        }
        playerToIndex[player] = players.length;
        players.push(player);
      }
      return true;
    },
    removePlayer: function removePlayer(player) {
      var index = playerToIndex[player];
      if (index === undefined) {
        return false;
      }

      playerToIndex[player] = undefined;
      players[index] = undefined;
      return true;
    },
    getPlayerIndex: function getPlayerIndex(player) {
      var index = playerToIndex[player];
      return index === undefined ? -1 : index;
    },
    getPlayers: function getPlayers() {
      return players.slice();
    },
    getNeighbours: function getNeighbours(player) {
      var index = playerToIndex[player];
      if (index === undefined) {
        return []; // player not in network
      }

      return adjList[index].map(function (i) {
        return players[i];
      }).filter(function (e) {
        return e !== undefined;
      });
    }
  };
}

module.exports = {
  matrixToList: matrixToList,
  listToMatrix: listToMatrix,

  /**
   * @param {adjList} list
   */
  create: function create(list) {
    return createNetwork(list);
  },


  /**
   * @param {adjList} list
   */
  fromAdjecencyList: function fromAdjecencyList(list) {
    return this.create(list);
  },


  /**
   * @param {adjList} matrix
   */
  fromAdjecencyMatrix: function fromAdjecencyMatrix(matrix) {
    var adjList = matrixToList(matrix);
    return this.fromAdjecencyList(adjList);
  },


  /* standard networks */
  /**
   * @param {number} playerCount
   */
  solo: function solo(playerCount) {
    return this.groups(playerCount, 1);
  },


  /**
   * @param {number} playerCount
   */
  pairs: function pairs(playerCount) {
    return this.groups(playerCount, 2);
  },


  /**
   * @param {number} playerCount
   */
  clique: function clique(playerCount) {
    return this.groups(playerCount, playerCount);
  },


  /**
   * @param {number} playerCount
   * @param {number} groupSize
   */
  groups: function groups(playerCount, groupSize) {
    var adjList = [];
    for (var i = 0; i < playerCount; i) {
      // create the group
      var group = [];
      for (var j = i; j < i + groupSize; j++) {
        group.push(j);
      }

      // and filter out players themselves

      var _loop = function _loop(k) {
        adjList.push(group.filter(function (n) {
          return n !== k;
        }));
      };

      for (var k = i; k < i + groupSize; k++) {
        _loop(k);
      }

      // go to next group
      i += groupSize;
    }
    return createNetwork(adjList);
  },


  /**
   * @param {number} playerCount
   * @param {number} noOfNeighbours
   */
  circular: function circular(playerCount, noOfNeighbours) {
    var adjList = [];
    for (var i = 0; i < playerCount; i++) {
      var neighbours = [];
      for (var j = i - noOfNeighbours; j <= i + noOfNeighbours; j++) {
        if (j === i) {
          continue;
        }

        if (j < 0) {
          neighbours.push(playerCount + j);
        } else if (j >= playerCount) {
          neighbours.push(j - playerCount);
        } else {
          neighbours.push(j);
        }
      }
      adjList.push(neighbours);
    }

    return createNetwork(adjList);
  }
};