'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* globals describe it expect jest beforeEach */

/**
 * @jest-environment node
 */

var _stages = require('./stages');

var _stages2 = _interopRequireDefault(_stages);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Define some stages
 */
var regularStage = {
  serverSide: {
    setup: jest.fn(),
    teardown: jest.fn()
  },
  clientSide: {
    setup: jest.fn(),
    teardown: jest.fn()
  },
  options: {}
};

var timedStage = _extends({}, regularStage, {
  options: {
    duration: 100,
    timeOnServer: true
  }
});
var clientTimedStage = _extends({}, timedStage, {
  options: {
    duration: 100
  }
});

var createOptions = {
  getContext: jest.fn(function () {
    return {
      getCanvas: function getCanvas() {
        return {
          remove: function remove() {},
          getObjects: function getObjects() {
            return [];
          }
        };
      }
    };
  }),
  getPlayers: function getPlayers() {
    return ['1', '2'];
  }, // fake two players
  onStageStarted: jest.fn(),
  onStageEnded: jest.fn(),
  onGameOver: jest.fn()
};

describe('manager', function () {
  beforeEach(function () {
    createOptions.onStageStarted.mockClear();
    createOptions.onStageEnded.mockClear();
    createOptions.onGameOver.mockClear();
    regularStage.serverSide.setup.mockClear();
    regularStage.serverSide.teardown.mockClear();
    regularStage.clientSide.setup.mockClear();
    regularStage.clientSide.teardown.mockClear();
  });

  it('start should start the first stage', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [regularStage, timedStage]
    }));
    man.start();
    expect(createOptions.onStageStarted).toBeCalledWith(0);
  });

  it('extraneous start calls should be ignored', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [regularStage, timedStage]
    }));
    man.start();
    expect(createOptions.onStageStarted).toBeCalledWith(0);
    man.start();
    man.start();
    expect(createOptions.onStageEnded).not.toBeCalledWith(0);
  });

  it('next should proceed to next stage', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [regularStage, timedStage]
    }));
    man.start();
    man.next();
    expect(createOptions.onStageEnded).toBeCalledWith(0);
    expect(createOptions.onStageStarted).toBeCalledWith(1);
  });

  it('currentStage should return current stage #', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [regularStage, timedStage]
    }));
    expect(man.getCurrentStage()).toBe(-1);
    man.start();
    expect(man.getCurrentStage()).toBe(0);
    man.next();
    expect(man.getCurrentStage()).toBe(1);
  });

  it('next results in game over when no more stages', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [regularStage, timedStage]
    }));
    man.start();
    man.next();
    man.next(); // no more
    expect(createOptions.onGameOver).toBeCalled();
  });

  it('after game over, nothing further happens', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [regularStage, timedStage]
    }));
    man.start();
    man.next();
    man.next();
    man.start(); // extraneous start
    man.next(); // and next
    expect(createOptions.onGameOver).toBeCalled();
    expect(man.getCurrentStage()).toBe(2); // not reset
  });

  it('server-timed stage finishes after duration', function (done) {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [timedStage]
    }));
    man.start();
    setTimeout(function () {
      expect(createOptions.onStageStarted).toBeCalledWith(0);
      expect(createOptions.onStageEnded).toBeCalledWith(0);
      done();
    }, 110);
  });

  it('server-timed stage can be terminated early', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [timedStage]
    }));
    man.start();
    man.next();
    expect(createOptions.onStageStarted).toBeCalledWith(0);
    expect(createOptions.onStageEnded).toBeCalledWith(0);
  });

  it('client-timed stage can be terminated early', function () {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [clientTimedStage]
    }));
    man.start();
    man.next();
    expect(createOptions.onStageStarted).toBeCalledWith(0);
    expect(createOptions.onStageEnded).toBeCalledWith(0);
  });

  it('client-timed stage terminates after duration', function (done) {
    var man = (0, _stages2.default)(_extends({}, createOptions, {
      stages: [clientTimedStage]
    }));
    man.start();
    setTimeout(function () {
      expect(createOptions.onStageEnded).toBeCalledWith(0);
      done();
    }, 110);
  });
});